import math
import time
import pandas as pd

from typing import Union, Optional, Tuple

from arize import public_pb2 as public__pb2
from arize.utils.types import ModelTypes
from google.protobuf.timestamp_pb2 import Timestamp

MAX_BYTES_PER_BULK_RECORD = 100000
MAX_DAYS_WITHIN_RANGE = 365


def infer_model_type(
    label: Union[str, bool, int, float, Tuple[str, float]]
) -> ModelTypes:
    val = convert_element(label)
    if isinstance(val, bool):
        return ModelTypes.BINARY
    elif isinstance(val, (int, float)):
        return ModelTypes.NUMERIC
    elif isinstance(val, str):
        return ModelTypes.CATEGORICAL
    elif isinstance(val, tuple):
        return ModelTypes.SCORE_CATEGORICAL
    raise TypeError(
        f"label {label} has type {type(label)}, but must be one of str, bool, float, or int"
    )


def validate_prediction_timestamps(prediction_ids, prediction_timestamps):
    if prediction_timestamps is None:
        return
    else:
        expected_count = prediction_ids.shape[0]

    if isinstance(prediction_timestamps, pd.Series):
        if prediction_timestamps.shape[0] != expected_count:
            raise ValueError(
                f"prediction_timestamps has {prediction_timestamps.shape[0]} elements, but must have same number of "
                f"elements as prediction_ids: {expected_count}. "
            )
    elif isinstance(prediction_timestamps, list):
        if len(prediction_timestamps) != expected_count:
            raise ValueError(
                f"prediction_timestamps has length {len(prediction_timestamps)} but must have same number of elements as "
                f"prediction_ids: {expected_count}. "
            )
    else:
        raise TypeError(
            f"prediction_timestamps is type {type(prediction_timestamps)}, but expected one of: pd.Series, list<int>"
        )
    now = int(time.time())
    for ts in prediction_timestamps:
        if not is_timestamp_in_range(now, ts):
            raise ValueError(
                f"timestamp: {ts} in prediction_timestamps is out of range. Value must be within 1 year of the current time."
            )


def num_chunks(records):
    total_bytes = sum(r.ByteSize() for r in records)
    num_of_bulk = math.ceil(total_bytes / MAX_BYTES_PER_BULK_RECORD)
    return math.ceil(len(records) / num_of_bulk)


def bundle_records(records) -> {}:
    recs_per_msg = num_chunks(records)
    recs = {
        (i, i + recs_per_msg): records[i : i + recs_per_msg]
        for i in range(0, len(records), recs_per_msg)
    }
    return recs


def get_bulk_records(
    organization_key: str, model_id: str, model_version: Optional[str], records
):
    for k, r in records.items():
        records[k] = public__pb2.BulkRecord(
            records=r,
            organization_key=organization_key,
            model_id=model_id,
            model_version=model_version,
        )
    return records


def convert_element(value):
    """converts scalar or array to python native"""
    val = getattr(value, "tolist", lambda: value)()
    # Check if it's a list since elements from pd indices are converted to a scalar
    # whereas pd series/dataframe elements are converted to list of 1 with the native value
    if isinstance(val, list):
        val = val[0]
    if pd.isna(val):
        return None
    return val


def get_value_object(name: str, value):
    if isinstance(value, public__pb2.Value):
        return value
    val = convert_element(value)
    if val is None:
        return None
    if isinstance(val, (str, bool)):
        return public__pb2.Value(string=str(val))
    if isinstance(val, int):
        return public__pb2.Value(int=val)
    if isinstance(val, float):
        return public__pb2.Value(double=val)
    else:
        raise TypeError(
            f'feature "{name}" = {value} is type {type(value)}, but must be one of: bool, str, float, int.'
        )


def get_timestamp(time_overwrite):
    if time_overwrite is None:
        return None
    time = convert_element(time_overwrite)
    if not isinstance(time_overwrite, int):
        raise TypeError(
            f"time_overwrite {time_overwrite} is type {type(time_overwrite)}, but expects int. (Unix epoch "
            f"time in seconds) "
        )
    ts = Timestamp()
    ts.FromSeconds(time)
    return ts


def is_timestamp_in_range(now: int, ts: int):
    max_time = now + (MAX_DAYS_WITHIN_RANGE * 24 * 60 * 60)
    min_time = now - (MAX_DAYS_WITHIN_RANGE * 24 * 60 * 60)
    if ts > max_time:
        return False
    if ts < min_time:
        return False
    return True



import os
import uuid
import time
import pandas as pd
import numpy as np

from arize.utils.types import ModelTypes, Environments
from arize.pandas.logger import Client, Schema

ITERATIONS = 1
NUM_RECORDS = 1000

client = Client(
    organization_key=os.environ.get("ARIZE_ORG_KEY"),
    api_key=os.environ.get("ARIZE_API_KEY"),
    uri="https://devr.arize.com/v1",
)

features = pd.DataFrame(
    np.random.randint(0, 100000000, size=(NUM_RECORDS, 12)),
    columns=list("ABCDEFGHIJKL"),
)
pred_labels = pd.DataFrame(np.random.randint(0, 100000000, size=(NUM_RECORDS, 1)), columns=["prediction_label"])
ids = pd.DataFrame([str(uuid.uuid4()) for _ in range(NUM_RECORDS)], columns=["prediction_id"])
inferences = pd.concat([features, pred_labels, ids], axis=1)

start = time.time_ns()
res = client.log(
    inferences,
    "/tmp/inferences.bin",
    "model_id",
    "model_version",
    None, ModelTypes.NUMERIC,
    Environments.PRODUCTION,
    Schema(prediction_id_column_name="prediction_id",
           feature_column_names=inferences.columns.drop("prediction_label", "prediction_id"),
           prediction_label_column_name="prediction_label"))

print(f"future completed with response code {res.status_code}")
if res.status_code != 200:
    print(f"future failed with response code {res.status_code}, {res.text}")

end = time.time_ns()
print(
    f"request took a total of {int(end - start)/1000000}ms to serialize and send {NUM_RECORDS} records.\n"
)



import json
import time

import pandas as pd
import concurrent.futures as cf
from typing import Union, Optional, Dict, List, Tuple
from google.protobuf.json_format import MessageToDict
from requests_futures.sessions import FuturesSession

from arize import public_pb2 as public__pb2
from arize.bounded_executor import BoundedExecutor
from arize.utils.types import ModelTypes
from arize.model import (
    TrainingRecords,
    ValidationRecords,
)
from arize.utils.utils import (
    validate_prediction_timestamps,
    bundle_records,
    convert_element,
    get_value_object,
    get_timestamp,
    is_timestamp_in_range,
    infer_model_type,
    get_bulk_records,
)

from arize.__init__ import __version__


def _label_validation(
    model_type: ModelTypes, label: Union[str, bool, int, float, Tuple[str, float]]
):
    if model_type == ModelTypes.BINARY:
        if not (isinstance(label, bool) or label == 0 or label == 1):
            raise TypeError(
                f"label {label} has type {type(label)}, but must be one a bool, 0 or 1 for ModelTypes.BINARY"
            )
    elif model_type == ModelTypes.NUMERIC:
        if not isinstance(label, (float, int)):
            raise TypeError(
                f"label {label} has type {type(label)}, but must be either float or int for ModelTypes.NUMERIC"
            )
    elif model_type == ModelTypes.CATEGORICAL:
        if not isinstance(label, str):
            raise TypeError(
                f"label {label} has type {type(label)}, but must be str for ModelTypes.CATEGORICAL"
            )
    elif model_type == ModelTypes.SCORE_CATEGORICAL:
        c = isinstance(label, str) or (
            isinstance(label, tuple)
            and isinstance(label[0], str)
            and isinstance(label[1], float)
        )
        if not c:
            raise TypeError(
                f"label {label} has type {type(label)}, but must be str or Tuple[str, float] for ModelTypes.SCORE_CATEGORICAL"
            )


def _get_label(
    name: str,
    value: Union[str, bool, int, float, Tuple[str, float]],
    model_type: Optional[ModelTypes],
) -> public__pb2.Label:
    if isinstance(value, public__pb2.Label):
        return value
    value = convert_element(value)
    if model_type == ModelTypes.SCORE_CATEGORICAL:
        if isinstance(value, tuple):
            return public__pb2.Label(
                score_categorical=public__pb2.ScoreCategorical(
                    categorical=value[0],
                    score=value[1],
                )
            )
        else:
            return public__pb2.Label(
                score_categorical=public__pb2.ScoreCategorical(categorical=value)
            )
    elif model_type == ModelTypes.BINARY:
        return public__pb2.Label(binary=value)
    elif model_type == ModelTypes.NUMERIC:
        return public__pb2.Label(numeric=value)
    elif model_type == ModelTypes.CATEGORICAL:
        return public__pb2.Label(categorical=value)
    raise TypeError(
        f"{name}_label = {value} of type {type(value)}. Must be one of str, bool, float, int, or Tuple[str, float]"
    )


def _validate_bulk_prediction(
    model_version,
    prediction_labels,
    prediction_ids,
    features,
    feature_names_overwrite,
    prediction_timestamps,
):
    if not isinstance(model_version, str):
        raise TypeError(
            f"model_version {model_version} is type {type(model_version)}, but must be a str"
        )

        # Validate prediction labels type and shape and that column length is the same as prediction ids
    if not isinstance(prediction_labels, (pd.DataFrame, pd.Series)):
        raise TypeError(
            f"prediction_labels is type {type(prediction_labels)}, but expects one of: pd.DataFrame, pd.Series"
        )
    if isinstance(prediction_labels, pd.DataFrame) and not (
        prediction_labels.shape[1] == 1 or prediction_labels.shape[1] == 2
    ):
        raise ValueError(
            f"prediction_labels contains {prediction_labels.shape[1]} columns, but can only have 1 or 2"
        )

    if isinstance(prediction_labels, pd.DataFrame) and prediction_labels.shape[1] == 2:
        if not pd.api.types.is_string_dtype(
            prediction_labels[prediction_labels.columns[0]]
        ):
            raise TypeError(
                f"Two column prediction_labels must have strings for column 0."
            )
        if not pd.api.types.is_numeric_dtype(
            prediction_labels[prediction_labels.columns[1]]
        ):
            raise TypeError(
                f"Two column prediction_labels must have numerics for column 1."
            )

    if prediction_labels.shape[0] != prediction_ids.shape[0]:
        raise ValueError(
            f"prediction_labels contains {prediction_labels.shape[0]} elements, but must have the same as "
            f"predictions_ids: {prediction_ids.shape[0]}. "
        )

        # Validate features type, shape matches prediction ids, and handle feature names overwrite
    if features is not None:
        if not isinstance(features, pd.DataFrame):
            raise TypeError(
                f"features is type {type(features)}, but expect type pd.DataFrame."
            )
        if features.shape[0] != prediction_ids.shape[0]:
            raise ValueError(
                f"features has {features.shape[0]} sets of features, but must match size of predictions_ids: "
                f"{prediction_ids.shape[0]}. "
            )
        if feature_names_overwrite is not None:
            if len(features.columns) != len(feature_names_overwrite):
                raise ValueError(
                    f"feature_names_overwrite has len:{len(feature_names_overwrite)}, but expects the same "
                    f"number of columns in features dataframe: {len(features.columns)}. "
                )
        else:
            if isinstance(features.columns, pd.core.indexes.numeric.NumericIndex):
                raise TypeError(
                    f"features.columns is of type {type(features.columns)}, but expect elements to be str. "
                    f"Alternatively, feature_names_overwrite must be present. "
                )
            for name in features.columns:
                if not isinstance(name, str):
                    raise TypeError(
                        f"features.column {name} is type {type(name)}, but expect str"
                    )

    # Validate timestamp overwrite
    validate_prediction_timestamps(prediction_ids, prediction_timestamps)


class Client:
    """
    Arize API Client to report model predictions and actuals to Arize AI platform
    """

    def __init__(
        self,
        api_key: str,
        organization_key: str,
        uri="https://api.arize.com/v1",
        max_workers=8,
        max_queue_bound=5000,
        retry_attempts=3,
        timeout=200,
    ):
        """
        :params api_key: (str) api key associated with your account with Arize AI
        :params organization_key: (str) organization key in Arize AI
        :params max_workers: (int) number of max concurrent requests to Arize. Default: 20
        :max_queue_bound: (int) number of maximum concurrent future objects being generated for publishing to Arize. Default: 5000
        """

        if not isinstance(organization_key, str):
            raise TypeError(
                f"organization_key {organization_key} is type {type(organization_key)}, but must be a str"
            )
        self._retry_attempts = retry_attempts
        self._uri = uri + "/log"
        self._bulk_url = uri + "/bulk"
        self._stream_uri = uri + "/preprod"
        self._files_uri = uri + "/files"
        self._api_key = api_key
        self._organization_key = organization_key
        self._timeout = timeout
        self._session = FuturesSession(
            executor=BoundedExecutor(max_queue_bound, max_workers)
        )
        # Grpc-Metadata prefix is required to pass non-standard md through via grpc-gateway
        self._header = {
            "authorization": api_key,
            "Grpc-Metadata-organization": organization_key,
            "Grpc-Metadata-sdk-version": __version__,
            "Grpc-Metadata-sdk": "py",
        }

    def log(
        self,
        model_id: str,
        prediction_id: str,
        model_version: str = None,
        prediction_label: Union[str, bool, int, float, Tuple[str, float]] = None,
        actual_label: Union[str, bool, int, float] = None,
        shap_values: Dict[str, float] = None,
        features: Optional[Dict[str, Union[str, bool, float, int]]] = None,
        model_type: Optional[ModelTypes] = None,
        prediction_timestamp: Optional[int] = None,
    ) -> cf.Future:
        """Logs a record to Arize via a POST request. Returns :class:`Future` object.
        :param model_id: (str) Unique identifier for a given model
        :param prediction_id: (str) Unique string identifier for a specific prediction. This value is used to match a prediction to an actual label or feature imporances in the Arize platform.
        :param model_version: (str) Field used to group together a subset of predictions and actuals for a given model_id.
        :param prediction_label: (one of str, bool, int, float, Tuple[str, float]) The predicted value for a given model input.
        :param actual_label: (one of str, bool, int, float) The actual true value for a given model input. This actual will be matched to the prediction with the same prediction_id as the one in this call.
        :param shap_values: (str, float) Dictionary containing human readable and debuggable model features keys, along with SHAP feature importance values. Keys must be str, while values must be float.
        :param features: (str, <value>) Optional dictionary containing human readable and debuggable model features. Keys must be str, values one of str, bool, float, long.
        :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
        :param prediction_timestamp: (int) Optional field with unix epoch time in seconds to overwrite timestamp for prediction. If None, prediction uses current timestamp.
        :rtype : concurrent.futures.Future
        """

        # Validate model_id
        if not isinstance(model_id, str):
            raise TypeError(
                f"model_id {model_id} is type {type(model_id)}, but must be a str"
            )

        # Validate feature types
        if features is not None and bool(features):
            for k, v in features.items():
                val = convert_element(v)
                if val is not None and not isinstance(val, (str, bool, float, int)):
                    raise TypeError(
                        f"feature {k} with value {v} is type {type(v)}, but expected one of: str, bool, float, int"
                    )

        # Check the timestamp present on the event
        if prediction_timestamp is not None and not isinstance(
            prediction_timestamp, int
        ):
            raise TypeError(
                f"prediction_timestamp {prediction_timestamp} is type {type(prediction_timestamp)} but expected int"
            )

        now = int(time.time())
        if prediction_timestamp is not None and not is_timestamp_in_range(
            now, prediction_timestamp
        ):
            raise ValueError(
                f"prediction_timestamp: {prediction_timestamp} is out of range. Value must be within 1 year of the current time."
            )

        # Construct the prediction
        p = None
        if prediction_label is not None:
            if not isinstance(model_version, str):
                raise TypeError(
                    f"model_version {model_version} is type {type(model_version)}, but must be a str"
                )
            model_type = (
                infer_model_type(prediction_label) if model_type is None else model_type
            )
            _label_validation(model_type, label=convert_element(prediction_label))
            p = public__pb2.Prediction(
                label=_get_label(
                    value=prediction_label,
                    name="prediction",
                    model_type=model_type,
                ),
                model_version=model_version,
            )
            if features is not None:
                converted_feats = {}
                for (k, v) in features.items():
                    val = get_value_object(value=v, name=k)
                    if val is not None:
                        converted_feats[k] = val
                feats = public__pb2.Prediction(features=converted_feats)
                p.MergeFrom(feats)
            if prediction_timestamp is not None:
                p.timestamp.MergeFrom(get_timestamp(prediction_timestamp))

        # Validate and construct the optional actual
        a = None
        if actual_label is not None:
            model_type = (
                infer_model_type(actual_label) if model_type is None else model_type
            )
            _label_validation(model_type, label=convert_element(actual_label))
            a = public__pb2.Actual(
                label=_get_label(
                    value=actual_label, name="actual", model_type=model_type
                )
            )

        # Validate and construct the optional feature importances
        fi = None
        if shap_values is not None and bool(shap_values):
            for k, v in shap_values.items():
                if not isinstance(convert_element(v), float):
                    raise TypeError(
                        f"feature {k} with value {v} is type {type(v)}, but expected one of: float"
                    )
            fi = public__pb2.FeatureImportances(feature_importances=shap_values)

        if p is None and a is None and fi is None:
            raise ValueError(
                f"must provide at least one of prediction_label, actual_label, or shap_values"
            )

        rec = public__pb2.Record(
            organization_key=self._organization_key,
            model_id=model_id,
            prediction_id=prediction_id,
            prediction=p,
            actual=a,
            feature_importances=fi,
        )
        return self._post(record=rec, uri=self._uri, indexes=None)

    def bulk_log(
        self,
        model_id: str,
        prediction_ids: Union[pd.DataFrame, pd.Series],
        model_version: str = None,
        prediction_labels: Union[
            pd.DataFrame, pd.Series
        ] = None,  # 1xN or 2xN (for scored categorical)
        features: Optional[Union[pd.DataFrame, pd.Series]] = None,
        actual_labels: Union[pd.DataFrame, pd.Series] = None,
        shap_values: Union[pd.DataFrame, pd.Series] = None,
        model_type: Optional[ModelTypes] = None,
        feature_names_overwrite: Optional[List[str]] = None,
        prediction_timestamps: Optional[Union[List[int], pd.Series]] = None,
    ) -> List[cf.Future]:
        """Logs a collection of predictions with Arize via a POST request. Returns list<:class:`Future`> object.
        :param model_id: (str) Unique identifier for a given model
        :param model_version: (str) Field used to group together a subset of predictions and actuals for a given model_id.
        :param prediction_ids: Pandas DataFrame with shape (N, 1) or Series with str valued elements. Each element corresponding to a unique string identifier for a specific prediction. These values are needed to match latent actual labels to their original prediction labels. Each element corresponds to feature values of the same index.
        :param prediction_labels: Optional Pandas DataFrame with shape (N, 1) or (N, 2) or Series. The predicted values for a given model input. Values are associates to the ids in the same index.  For a (N, 2) DataFrame column 0 is interpretted as the prediction category and column 1 is interpretted as the prediction score.
        :param features: Optional Pandas DataFrame with shape (N, 2) containing human readable and debuggable model features. DataFrames columns (df.columns) should contain feature names and must have same number of rows as prediction_ids and prediction_labels. N.B. np.nan values are stripped from the record and manifest on our platform as a missing value (not 0.0 or NaN)
        :param actual_labels: Optional Pandas DataFrame with shape (N, 1) or Series. The actual true values for a given model input. Values are associates to the labels in the same index.
        :param shap_values: Optional Pandas DataFrame with shape (N, 1) or Series. The SHAP value sets for a set of predictions. SHAP value sets are correspond to the prediction ids with the same index.
        :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
        :param feature_names_overwrite: Optional list<str> that if present will overwrite features.columns values. Must contain the same number of elements as features.columns.
        :param prediction_timestamps: (list<int>) Optional list with same number of elements as prediction_labels field with unix epoch time in seconds to overwrite timestamp for each prediction. If None, prediction uses current timestamp.
        :rtype : list<concurrent.futures.Future>
        """

        # Validate model_id
        if not isinstance(model_id, str):
            raise TypeError(
                f"model_id {model_id} is type {type(model_id)}, but must be a str"
            )

        # Validate prediction_ids
        if not isinstance(prediction_ids, (pd.DataFrame, pd.Series)):
            raise TypeError(
                f"prediction_ids is type {type(prediction_ids)}, but expect one of: pd.DataFrame, pd.Series"
            )

        if prediction_labels is not None:
            _validate_bulk_prediction(
                model_version,
                prediction_labels,
                prediction_ids,
                features,
                feature_names_overwrite,
                prediction_timestamps,
            )
            model_type = (
                infer_model_type(prediction_labels.iloc[0])
                if model_type is None
                else model_type
            )

        if actual_labels is not None:
            if not isinstance(actual_labels, (pd.DataFrame, pd.Series)):
                raise TypeError(
                    f"actual_labels is type: {type(actual_labels)}, but expects one of: pd.DataFrame, pd.Series"
                )
            if actual_labels.shape[0] != prediction_ids.shape[0]:
                raise ValueError(
                    f"actual_labels contains {actual_labels.shape[0]} elements, but must have the same as "
                    f"predictions_ids: {prediction_ids.shape[0]}. "
                )
            # Set model type if not yet set
            model_type = (
                infer_model_type(actual_labels.iloc[0])
                if model_type is None
                else model_type
            )

        if shap_values is not None:
            if not isinstance(shap_values, pd.DataFrame):
                raise TypeError(
                    f"shap_values is type {type(shap_values)}, but expect type pd.DataFrame."
                )
            if shap_values.shape[0] != prediction_ids.shape[0]:
                raise ValueError(
                    f"shap_values has {shap_values.shape[0]} sets of values, but must match size of "
                    f"predictions_ids: {shap_values.shape[0]}. "
                )
            if isinstance(shap_values.columns, pd.core.indexes.numeric.NumericIndex):
                raise TypeError(
                    f"shap_values.columns is of type {type(shap_values.columns)}, but expect elements to be str."
                )
            for name in shap_values.columns:
                if not isinstance(name, str):
                    raise TypeError(
                        f"shap_values.column {name} is type {type(name)}, but expect str"
                    )

        prediction_ids = prediction_ids.to_numpy()
        prediction_labels = (
            prediction_labels.to_numpy() if prediction_labels is not None else None
        )
        prediction_timestamps = (
            prediction_timestamps.tolist()
            if isinstance(prediction_timestamps, pd.Series)
            else prediction_timestamps
        )
        if features is not None:
            feature_names = feature_names_overwrite or features.columns
            features = features.to_numpy()
        actual_labels = actual_labels.to_numpy() if actual_labels is not None else None
        shap_columns = shap_values.columns if shap_values is not None else None
        shap_values = shap_values.to_numpy() if shap_values is not None else None

        records = []
        for row, v in enumerate(prediction_ids):
            pred_id = v if isinstance(v, str) else v[0]
            if not isinstance(pred_id, (str, bytes)):
                raise TypeError(
                    f"prediction_id {pred_id} is type {type(pred_id)}, but expected one of: str, bytes"
                )
            p = None
            if prediction_labels is not None:
                # if there is more than 1 dimension, and the second dimension size is 2 - TODO instead just guarantee shape is always (X,Y) instead of sometimes (X,)
                if (
                    len(prediction_labels.shape) == 2
                    and prediction_labels.shape[1] == 2
                ):
                    label = _get_label(
                        value=(prediction_labels[row][0], prediction_labels[row][1]),
                        name="prediction",
                        model_type=model_type,
                    )
                else:
                    label = _get_label(
                        value=prediction_labels[row],
                        name="prediction",
                        model_type=model_type,
                    )
                p = public__pb2.Prediction(label=label)
                if features is not None:
                    converted_feats = {}
                    for column, name in enumerate(feature_names):
                        val = get_value_object(value=features[row][column], name=name)
                        if val is not None:
                            converted_feats[name] = val
                    feats = public__pb2.Prediction(features=converted_feats)
                    p.MergeFrom(feats)
                if prediction_timestamps is not None:
                    p.timestamp.MergeFrom(get_timestamp(prediction_timestamps[row]))

            a = None
            if actual_labels is not None:
                a = public__pb2.Actual(
                    label=_get_label(
                        value=actual_labels[row], name="actual", model_type=model_type
                    )
                )

            fi = None
            if shap_values is not None:
                converted_fi = {
                    name: shap_values[row][column]
                    for column, name in enumerate(shap_columns)
                }
                fi = public__pb2.FeatureImportances(feature_importances=converted_fi)

            rec = public__pb2.Record(
                prediction_id=pred_id,
                prediction=p,
                actual=a,
                feature_importances=fi,
            )
            records.append(rec)

        brs = get_bulk_records(
            self._organization_key, model_id, model_version, bundle_records(records)
        )
        return self._post_bulk(records=brs, uri=self._bulk_url)

    def log_validation_records(
        self,
        model_id: str,
        model_version: str,
        batch_id: str,
        prediction_labels: Union[pd.DataFrame, pd.Series],
        actual_labels: Union[pd.DataFrame, pd.Series],
        prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
        prediction_ids: Optional[Union[pd.DataFrame, pd.Series]] = None,
        model_type: Optional[ModelTypes] = None,
        features: Optional[Union[pd.DataFrame, pd.Series]] = None,
        prediction_timestamps: Optional[Union[List[int], pd.Series]] = None,
    ) -> List[cf.Future]:
        """Logs a set of validation records to Arize. Returns :class:`Future` object.
        :param model_id: (str) Unique identifier for a given model.
        :param model_type: (ModelTypes) Declares what model type these records are for. Binary, Numeric, Categorical, Score_Categorical.
        :param model_version: (str) Unique identifier used to group together a subset of records for a given model_id.
        :param batch_id: (str) Unique identifier used to group together a subset of validation records for a given model_id and model_version - akin to a validation set.
        :param prediction_labels: 1-D Pandas DataFrame or Series. The predicted values for a given model input.
        :param actual_labels: 1-D Pandas DataFrame or Series. The actual true values for a given model input.
        :param prediction_scores: 1-D Pandas DataFrame or Series. The predicted scores for the corresponding predicted_label of classification model. If present, elements in prediction_labels must be of type str. Values are associates to the labels in the same index.
        :param prediction_ids: 1-D Pandas DataFrame or Series. The prediction IDs for the corresponding predicted_label of a classification_model. If present, elements in prediction_labels must be of type str. Values are associates to the labels in the same index.
        :param features: Optional 2-D Pandas DataFrame containing human readable and debuggable model features. DataFrames columns (df.columns) should contain feature names and must have same number of rows as actual_labels and prediction_labels. N.B. np.nan values are stripped from the record and manifest on our platform as a missing value (not 0.0 or NaN)
        :param prediction_timestamps: (list<int>) Optional list with same number of elements as prediction_labels field with unix epoch time in seconds to overwrite timestamp for each prediction. If None, prediction uses current timestamp.
        :rtype : list<concurrent.futures.Future>
        """
        rec = ValidationRecords(
            organization_key=self._organization_key,
            model_id=model_id,
            model_type=model_type,
            model_version=model_version,
            batch_id=batch_id,
            features=features,
            prediction_labels=prediction_labels,
            actual_labels=actual_labels,
            prediction_scores=prediction_scores,
            prediction_ids=prediction_ids,
            prediction_timestamps=prediction_timestamps,
        )
        rec.validate_inputs()
        return self._post_preprod(records=rec.build_proto())

    def log_training_records(
        self,
        model_id: str,
        model_version: str,
        prediction_labels: Union[pd.DataFrame, pd.Series],
        actual_labels: Union[pd.DataFrame, pd.Series],
        prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
        model_type: Optional[ModelTypes] = None,
        features: Optional[Union[pd.DataFrame, pd.Series]] = None,
    ) -> List[cf.Future]:
        """Logs a stream of training records to Arize. Returns :class:`Future` object.
        :param model_id: (str) Unique identifier for a given model.
        :param model_version: (str) Unique identifier used to group together a subset of records for a given model_id.
        :param model_type: (ModelTypes) Declares what model type these records are for. Binary, Numeric, Categorical, Score_Categorical.
        :param prediction_labels: 1-D Pandas DataFrame or Series. The predicted values for a given model input.
        :param actual_labels: 1-D Pandas DataFrame or Series. The actual true values for a given model input.
        :param prediction_scores: 1-D Pandas DataFrame or Series. The predicted scores for the corresponding predicted_label of classification model. If present, elements in prediction_labels must be of type str. Values are associates to the labels in the same index.
        :param features: Optional 2-D Pandas DataFrame containing human readable and debuggable model features. DataFrames columns (df.columns) should contain feature names and must have same number of rows as actual_labels and prediction_labels. N.B. np.nan values are stripped from the record and manifest on our platform as a missing value (not 0.0 or NaN)
        :rtype : list<concurrent.futures.Future>
        """
        rec = TrainingRecords(
            organization_key=self._organization_key,
            model_id=model_id,
            model_type=model_type,
            model_version=model_version,
            features=features,
            prediction_labels=prediction_labels,
            prediction_scores=prediction_scores,
            actual_labels=actual_labels,
        )
        rec.validate_inputs()
        return self._post_preprod(records=rec.build_proto())

    # Deprecated
    def log_prediction(
        self,
        model_id: str,
        model_version: str,
        prediction_id: str,
        prediction_label: Union[str, bool, int, float],
        prediction_score: Optional[float] = None,
        features: Optional[Dict[str, Union[str, bool, float, int]]] = None,
        model_type: Optional[ModelTypes] = None,
        time_overwrite: Optional[int] = None,
    ) -> cf.Future:
        """Logs a prediction to Arize via a POST request. Returns :class:`Future` object.
        :param model_id: (str) Unique identifier for a given model
        :param model_version: (str) Field used to group together a subset of predictions and actuals for a given model_id.
        :param prediction_id: (str) Unique string identifier for a specific prediction. This value is used to match a prediction to an actual label in the Arize platform.
        :param prediction_label: (one of bool, str, float, int) The predicted value for a given model input.
        :param prediction_score: (float) Optional predicted score for the predicted_label of classification model. If present, the prediction_label must be of type str.
        :param features: (str, <value>) Optional dictionary containing human readable and debuggable model features. Keys must be str, values one of str, bool, float, long.
        :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
        :param time_overwrite: (int) Optional field with unix epoch time in seconds to overwrite timestamp for prediction. If None, prediction uses current timestamp.
        :rtype : concurrent.futures.Future
        """
        return self.log(
            model_id=model_id,
            prediction_id=prediction_id,
            model_version=model_version,
            prediction_label=prediction_label
            if prediction_score is None
            else (prediction_label, prediction_score),
            features=features,
            model_type=model_type,
            prediction_timestamp=time_overwrite,
        )

    # Deprecated
    def log_actual(
        self,
        model_id: str,
        prediction_id: str,
        actual_label: Union[str, bool, int, float],
        model_type: Optional[ModelTypes] = None,
    ) -> cf.Future:
        """Logs an actual to Arize via a POST request. Returns :class:`Future` object.
        :param model_id: (str) Unique identifier for a given model
        :param prediction_id: (str) Unique string identifier for a specific prediction. This value is used to match a prediction to an actual label in the Arize platform.
        :param actual_label: (one of bool, str, float, int) The actual true value for a given model input. This actual will be matched to the prediction with the same prediction_id as the one in this call.
        :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
        :rtype : concurrent.futures.Future
        """
        return self.log(
            model_id=model_id,
            model_type=model_type,
            prediction_id=prediction_id,
            actual_label=actual_label,
        )

    # Deprecated
    def log_shap_values(
        self,
        model_id: str,
        prediction_id: str,
        shap_values: Dict[str, float],
    ) -> cf.Future:
        """Logs SHAP feature importance values for a given prediction to Arize via a POST request. Returns :class:`Future` object.
        :param model_id: (str) Unique identifier for a given model.
        :param prediction_id: (str) Unique string identifier for a specific prediction. This value is used to match a prediction to the SHAP values supplied in this request in the Arize platform.
        :param shap_values: (str, float) Dictionary containing human readable and debuggable model features keys, along with SHAP feature importance values. Keys must be str, while values must be float.
        :rtype : concurrent.futures.Future
        """
        return self.log(
            prediction_id=prediction_id,
            model_id=model_id,
            shap_values=shap_values,
        )

    # Deprecated
    def log_bulk_shap_values(
        self,
        model_id: str,
        prediction_ids: Union[pd.DataFrame, pd.Series],
        shap_values: Union[pd.DataFrame, pd.Series],
    ) -> List[cf.Future]:
        """Logs a collection of SHAP feature importance value sets with Arize via a POST request. Returns list<:class:`Future`> object.
        :param model_id: (str) Unique identifier for a given model
        :param prediction_ids: 1-D Pandas DataFrame or Series with string elements. Each element corresponding to a unique string identifier for a specific prediction. Each element corresponds to the SHAP values of the same index.
        :param shap_values: 1-D Pandas DataFrame or Series. The SHAP value sets for a set of predictions. SHAP value sets are correspond to the prediction ids with the same index.
        :rtype : list<concurrent.futures.Future>
        """
        return self.bulk_log(
            model_id=model_id, prediction_ids=prediction_ids, shap_values=shap_values
        )

    # Deprecated
    def log_bulk_predictions(
        self,
        model_id: str,
        model_version: str,
        prediction_ids: Union[pd.DataFrame, pd.Series],
        prediction_labels: Union[pd.DataFrame, pd.Series],
        prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
        features: Optional[Union[pd.DataFrame, pd.Series]] = None,
        model_type: Optional[ModelTypes] = None,
        feature_names_overwrite: Optional[List[str]] = None,
        time_overwrite: Optional[List[int]] = None,
    ) -> List[cf.Future]:
        """Logs a collection of predictions with Arize via a POST request. Returns list<:class:`Future`> object.
        :param model_id: (str) Unique identifier for a given model
        :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
        :param model_version: (str) Field used to group together a subset of predictions and actuals for a given model_id.
        :param prediction_ids: 1-D Pandas DataFrame or Series with string elements. Each element corresponding to a unique string identifier for a specific prediction. These values are needed to match latent actual labels to their original prediction labels. Each element corresponds to feature values of the same index.
        :param prediction_labels: 1-D Pandas DataFrame or Series. The predicted values for a given model input. Values are associates to the ids in the same index.
        :param prediction_scores: 1-D Pandas DataFrame or Series. The predicted scores for the corresponding predicted_label of classification model. If present, elements in prediction_labels must be of type str. Values are associates to the labels in the same index.
        :param features: Optional 2-D Pandas DataFrame containing human readable and debuggable model features. DataFrames columns (df.columns) should contain feature names and must have same number of rows as prediction_ids and prediction_labels. N.B. np.nan values are stripped from the record and manifest on our platform as a missing value (not 0.0 or NaN)
        :param feature_names_overwrite: Optional list<str> that if present will overwrite features.columns values. Must contain the same number of elements as features.columns.
        :param time_overwrite: (list<int>) Optional list with same number of elements as prediction_labels field with unix epoch time in seconds to overwrite timestamp for each prediction. If None, prediction uses current timestamp.
        :rtype : list<concurrent.futures.Future>
        """
        return self.bulk_log(
            model_id=model_id,
            model_type=model_type,
            model_version=model_version,
            prediction_ids=prediction_ids,
            prediction_labels=pd.concat([prediction_labels, prediction_scores], axis=1),
            features=features,
            feature_names_overwrite=feature_names_overwrite,
            prediction_timestamps=time_overwrite,
        )

    # Deprecated
    def log_bulk_actuals(
        self,
        model_id: str,
        prediction_ids: Union[pd.DataFrame, pd.Series],
        actual_labels: Union[pd.DataFrame, pd.Series],
        model_type: Optional[ModelTypes] = None,
    ) -> List[cf.Future]:
        """Logs a collection of actuals with Arize via a POST request. Returns list<:class:`Future`> object.
        :param model_id: (str) Unique identifier for a given model
        :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
        :param prediction_ids: 1-D Pandas DataFrame or Series with string elements. Each element corresponding to a unique string identifier for a specific prediction. These values are needed to match latent actual labels to their original prediction labels. Each element corresponds to feature values of the same index.
        :param actual_labels: 1-D Pandas DataFrame or Series. The actual true values for a given model input. Values are associates to the labels in the same index.
        :rtype : list<concurrent.futures.Future>
        """
        return self.bulk_log(
            model_id=model_id,
            model_type=model_type,
            prediction_ids=prediction_ids,
            actual_labels=actual_labels,
        )

    def _post_bulk(self, records, uri):
        return [self._post(r, uri, k) for k, r in records.items()]

    def _post(self, record, uri, indexes):
        resp = self._session.post(
            uri,
            headers=self._header,
            timeout=self._timeout,
            json=MessageToDict(message=record, preserving_proto_field_name=True),
        )
        if indexes is not None and len(indexes) == 2:
            resp.starting_index = indexes[0]
            resp.ending_index = indexes[1]
        return resp

    def _post_preprod(self, records):
        futures = []
        for k, recs in records.items():
            futures.append(
                self._session.post(
                    self._stream_uri,
                    headers=self._header,
                    timeout=self._timeout,
                    data="\n".join(
                        json.dumps(
                            MessageToDict(message=d, preserving_proto_field_name=True)
                        )
                        for d in recs
                    ),
                )
            )
        return futures





from enum import Enum, unique

@unique
class ModelTypes(Enum):
    BINARY = 1
    NUMERIC = 2
    CATEGORICAL = 3
    SCORE_CATEGORICAL = 4

@unique
class Environments(Enum):
    PRODUCTION = 1
    VALIDATION = 2
    TRAINING = 3



import os
import uuid
import time
import pandas as pd
import numpy as np
import concurrent.futures as cf

from arize.api import Client
from arize.types import ModelTypes

ITERATIONS = 1
NUM_RECORDS = 100

arize = Client(
    organization_key=os.environ.get("ARIZE_ORG_KEY"),
    api_key=os.environ.get("ARIZE_API_KEY"),
)

features = pd.DataFrame(
    np.random.randint(0, 100000000, size=(NUM_RECORDS, 12)),
    columns=list("ABCDEFGHIJKL"),
)
pred_labels = pd.DataFrame(np.random.randint(0, 100000000, size=(NUM_RECORDS, 1)))
ids = pd.DataFrame([str(uuid.uuid4()) for _ in range(NUM_RECORDS)])
column_overwrite = list("abcdefghijkl")

start = time.time_ns()
preds = arize.bulk_log(
    model_id="example_model_id",
    model_version="v0.1",
    model_type=ModelTypes.NUMERIC,
    prediction_ids=ids,
    prediction_labels=pred_labels,
    features=features,
    feature_names_overwrite=column_overwrite,
    actual_labels=pred_labels,
)

end_enqueue = time.time_ns()
print(
    f"request took a total of {int(end_enqueue - start)/1000000}ms to enqueue. Waiting for responses.\n"
)

for future in cf.as_completed(preds):
    res = future.result()
    print(f"future completed with response code {res.status_code}")
    if res.status_code != 200:
        print(f"future failed with response code {res.status_code}, {res.text}")

end_sending = time.time_ns()
print(
    f"Process took a total of {int(end_sending - start)/1000000}ms to send {NUM_RECORDS} records."
)



import os
import time
import uuid
import numpy
from random import random
import concurrent.futures as cf

from arize.api import Client
from arize.types import ModelTypes

ITERATIONS = 1
NUM_FEATURES = 5

arize = Client(
    organization_key=os.environ.get("ARIZE_ORG_KEY"),
    api_key=os.environ.get("ARIZE_API_KEY"),
)


def get_features(feature_counts):
    features = {}
    for i in range(feature_counts):
        features["feature_" + str(i) + "_bool"] = True
        features["feature_" + str(i) + "_str"] = "str val"
        features["feature_" + str(i) + "_float"] = random()
        features["feature_" + str(i) + "_np"] = numpy.float_(random())
        features["feature_" + str(i) + "_np_ll"] = numpy.longlong(random() * 100)
    return features


def get_shap_values(features):
    shap_values = {}
    for feature in features:
        shap_values[feature] = numpy.float_(random())
    return shap_values


features = get_features(NUM_FEATURES)
shap_values = get_shap_values(features)
resps = []
start = time.time_ns()
for j in range(ITERATIONS):
    id_ = str(uuid.uuid4())
    pred = arize.log(
        model_id="example_model_id",
        model_type=ModelTypes.BINARY,
        model_version="v0.1",
        prediction_id=id_,
        prediction_label=True,
        features=features,
        shap_values=shap_values,
    )
    resps.append(pred)

end_sending = time.time_ns()
print(
    f"{ITERATIONS} requests took a total of {int(end_sending - start)/1000000}ms to send. Waiting for responses."
)

for future in cf.as_completed(resps):
    res = future.result()
    print(f"future completed with response code {res.status_code}")
    if res.status_code != 200:
        print(f"future failed with response code {res.status_code}, {res.text}")



import time
import requests
from typing import List, Dict
from dataclasses import dataclass
from arize import public_pb2 as pb
from arize.utils.types import ModelTypes, Environments


@dataclass(frozen=True)
class Schema:
    prediction_id_column_name: str
    feature_column_names: List[str]
    timestamp_column_name: str = None
    prediction_label_column_name: str = None
    prediction_score_column_name: str = None
    actual_label_column_name: str = None
    shap_values_column_names: Dict[str, str] = None


class Client:
    def __init__(
            self,
            api_key: str,
            organization_key: str,
            uri="https://api.arize.com/v1"):
        self._api_key = api_key
        self._organization_key = organization_key
        self._files_uri = uri + "/files"

    def log(
        self,
        dataframe,
        path: str,
        model_id: str,
        model_version: str,
        batch_id: str,
        model_type: ModelTypes,
        environment: Environments,
        schema: Schema
    ):
        col_idx = {col: idx for idx, col in enumerate(dataframe.columns)}
        h = pb.FileHeader()

        if model_type is None:
            raise AttributeError("model_type is required")

        if environment is None:
            raise AttributeError("environment is required")

        if environment is Environments.TRAINING:
            if schema.prediction_label_column_name is None or schema.actual_label_column_name is None:
                raise AttributeError("both prediction and actual label must be specified for Training environment")
            if schema.shap_values_column_names is not None:
                raise AttributeError("shap_values are not supported for Training environments")
            h.environment = pb.FileHeader.Environment.TRAINING
        elif environment is Environments.VALIDATION:
            if schema.prediction_label_column_name is None or schema.actual_label_column_name is None:
                raise AttributeError("both prediction and actual label must be specified for Validation environment")
            if schema.shap_values_column_names is not None:
                raise AttributeError("shap_values are not supported for Validation environments")
            if batch_id is None:
                raise AttributeError("batch_id is required for Validation environment")
            h.environment = pb.FileHeader.Environment.VALIDATION
        elif environment is Environments.PRODUCTION:
            h.environment = pb.FileHeader.Environment.PRODUCTION
        else:
            raise AttributeError(f"unknown environment {environment}")

        with open(path, "wb") as f:
            header = h.SerializeToString()
            f.write(len(header).to_bytes(8, "big", signed=False))
            f.write(header)

            current_time = int(time.time())
            for row in dataframe.to_numpy():
                if Environments.TRAINING == environment:
                    msg = pb.PreProductionRecord()
                    r = msg.training_record.record
                elif Environments.VALIDATION == environment:
                    msg = pb.PreProductionRecord()
                    msg.validation_record.batch_id = batch_id
                    r = msg.validation_record.record
                else:
                    msg = pb.Record()
                    r = msg

                r.prediction_id = row[col_idx[schema.prediction_id_column_name]]
                r.model_id = model_id

                t = (
                    int(row[col_idx[schema.timestamp_column_name]])
                    if schema.timestamp_column_name is not None
                    else current_time
                )

                for feature_cn in schema.feature_column_names:
                    row_val = row[col_idx[feature_cn]]
                    feature_val = r.prediction.features[feature_cn]
                    if isinstance(row_val, (str, bool)):
                        feature_val.string = row_val
                    elif isinstance(row_val, int):
                        feature_val.int = row_val
                    elif isinstance(row_val, float):
                        feature_val.double = row_val

                if schema.prediction_label_column_name is not None:
                    r.prediction.timestamp.seconds = t
                    r.prediction.model_version = model_version
                    if model_type is ModelTypes.SCORE_CATEGORICAL:
                        r.prediction.label.score_categorical.categorical = row[
                            col_idx[schema.prediction_label_column_name]
                        ]
                        r.prediction.label.score_categorical.score = row[
                            col_idx[schema.prediction_score_column_name]
                        ]
                    elif model_type is ModelTypes.CATEGORICAL:
                        r.prediction.label.categorical = row[
                            col_idx[schema.prediction_label_column_name]
                        ]
                    elif model_type is model_type.NUMERIC:
                        r.prediction.label.numeric = row[
                            col_idx[schema.prediction_label_column_name]
                        ]
                    elif model_type is model_type.BINARY:
                        r.prediction.label.binary = row[
                            col_idx[schema.prediction_label_column_name]
                        ]

                if schema.actual_label_column_name is not None:
                    if model_type is ModelTypes.CATEGORICAL:
                        r.actual.label.categorical = row[col_idx[schema.actual_label_column_name]]
                    elif model_type is ModelTypes.NUMERIC:
                        r.actual.label.numeric = row[col_idx[schema.actual_label_column_name]]
                    elif model_type is ModelTypes.BINARY:
                        r.actual.label.binary = row[col_idx[schema.actual_label_column_name]]
                    elif model_type is ModelTypes.SCORE_CATEGORICAL:
                        if isinstance(schema.actual_label_column_name, tuple):
                            r.actual.label.score_categorical.categorical = row[col_idx[schema.actual_label_column_name[0]]]
                            r.actual.label.score_categorical.score = row[col_idx[schema.actual_label_column_name[1]]]
                        else:
                            r.actual.label.score_categorical.categorical = row[col_idx[schema.actual_label_column_name]]

                if schema.shap_values_column_names is not None:
                    for feature_name, shap_values_cn in schema.shap_values_column_names.items():
                        row_val = row[col_idx[shap_values_cn]]
                        r.feature_importances.feature_importances[feature_name] = row_val

                msg_bytes = msg.SerializeToString()
                f.write(len(msg_bytes).to_bytes(8, "big", signed=False))
                f.write(msg_bytes)
        return self._post_file(path)

    def _post_file(self, path):
        with open(path, "rb") as f:
            return requests.post(
                self._files_uri,
                data=f,
                headers={
                    "authorization": self._api_key,
                    "organization": self._organization_key,
                },
            )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: public.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='public.proto',
  package='public',
  syntax='proto3',
  serialized_options=b'\n\022com.arize.protocolZ9github.com/Arize-ai/arize/go/pkg/receiver/protocol/public',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cpublic.proto\x12\x06public\x1a\x1fgoogle/protobuf/timestamp.proto\"\x81\x01\n\nBulkRecord\x12\x18\n\x10organization_key\x18\x01 \x01(\t\x12\x10\n\x08model_id\x18\x02 \x01(\t\x12\x15\n\rmodel_version\x18\x03 \x01(\t\x12\x1f\n\x07records\x18\x05 \x03(\x0b\x32\x0e.public.RecordJ\x04\x08\x04\x10\x05R\ttimestamp\"\xa0\x02\n\x06Record\x12\x18\n\x10organization_key\x18\x01 \x01(\t\x12\x10\n\x08model_id\x18\x02 \x01(\t\x12\x15\n\rprediction_id\x18\x03 \x01(\t\x12&\n\nprediction\x18\x08 \x01(\x0b\x32\x12.public.Prediction\x12\x1e\n\x06\x61\x63tual\x18\t \x01(\x0b\x32\x0e.public.Actual\x12\x37\n\x13\x66\x65\x61ture_importances\x18\n \x01(\x0b\x32\x1a.public.FeatureImportances\x12:\n\x15prediction_and_actual\x18\x0b \x01(\x0b\x32\x1b.public.PredictionAndActualJ\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08\"\xae\x02\n\x13PreProductionRecord\x12\x45\n\x0ftraining_record\x18\x01 \x01(\x0b\x32*.public.PreProductionRecord.TrainingRecordH\x00\x12I\n\x11validation_record\x18\x02 \x01(\x0b\x32,.public.PreProductionRecord.ValidationRecordH\x00\x1a\x44\n\x10ValidationRecord\x12\x10\n\x08\x62\x61tch_id\x18\x01 \x01(\t\x12\x1e\n\x06record\x18\x02 \x01(\x0b\x32\x0e.public.Record\x1a\x30\n\x0eTrainingRecord\x12\x1e\n\x06record\x18\x01 \x01(\x0b\x32\x0e.public.RecordB\r\n\x0brecord_type\"6\n\x10ScoreCategorical\x12\x13\n\x0b\x63\x61tegorical\x18\x01 \x01(\t\x12\r\n\x05score\x18\x02 \x01(\x01\"\x82\x01\n\x05Label\x12\x10\n\x06\x62inary\x18\x01 \x01(\x08H\x00\x12\x15\n\x0b\x63\x61tegorical\x18\x02 \x01(\tH\x00\x12\x11\n\x07numeric\x18\x03 \x01(\x01H\x00\x12\x35\n\x11score_categorical\x18\x04 \x01(\x0b\x32\x18.public.ScoreCategoricalH\x00\x42\x06\n\x04\x64\x61ta\"\xe4\x01\n\nPrediction\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12\x1c\n\x05label\x18\x03 \x01(\x0b\x32\r.public.Label\x12\x32\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32 .public.Prediction.FeaturesEntry\x1a>\n\rFeaturesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1c\n\x05value\x18\x02 \x01(\x0b\x32\r.public.Value:\x02\x38\x01\"m\n\x05Value\x12\x10\n\x06string\x18\x01 \x01(\tH\x00\x12\r\n\x03int\x18\x02 \x01(\x03H\x00\x12\x10\n\x06\x64ouble\x18\x03 \x01(\x01H\x00\x12)\n\x0bmulti_value\x18\x04 \x01(\x0b\x32\x12.public.MultiValueH\x00\x42\x06\n\x04\x64\x61ta\"\x1c\n\nMultiValue\x12\x0e\n\x06values\x18\x01 \x03(\t\"U\n\x06\x41\x63tual\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1c\n\x05label\x18\x02 \x01(\x0b\x32\r.public.Label\"\xe6\x01\n\x12\x46\x65\x61tureImportances\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12O\n\x13\x66\x65\x61ture_importances\x18\x03 \x03(\x0b\x32\x32.public.FeatureImportances.FeatureImportancesEntry\x1a\x39\n\x17\x46\x65\x61tureImportancesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"]\n\x13PredictionAndActual\x12&\n\nprediction\x18\x01 \x01(\x0b\x32\x12.public.Prediction\x12\x1e\n\x06\x61\x63tual\x18\x02 \x01(\x0b\x32\x0e.public.Actual\"\x8b\x01\n\nFileHeader\x12\x33\n\x0b\x65nvironment\x18\x01 \x01(\x0e\x32\x1e.public.FileHeader.Environment\"H\n\x0b\x45nvironment\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08TRAINING\x10\x01\x12\x0e\n\nVALIDATION\x10\x02\x12\x0e\n\nPRODUCTION\x10\x03\x42O\n\x12\x63om.arize.protocolZ9github.com/Arize-ai/arize/go/pkg/receiver/protocol/publicb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])



_FILEHEADER_ENVIRONMENT = _descriptor.EnumDescriptor(
  name='Environment',
  full_name='public.FileHeader.Environment',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRAINING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VALIDATION', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRODUCTION', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1829,
  serialized_end=1901,
)
_sym_db.RegisterEnumDescriptor(_FILEHEADER_ENVIRONMENT)


_BULKRECORD = _descriptor.Descriptor(
  name='BulkRecord',
  full_name='public.BulkRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='organization_key', full_name='public.BulkRecord.organization_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_id', full_name='public.BulkRecord.model_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_version', full_name='public.BulkRecord.model_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='records', full_name='public.BulkRecord.records', index=3,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=58,
  serialized_end=187,
)


_RECORD = _descriptor.Descriptor(
  name='Record',
  full_name='public.Record',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='organization_key', full_name='public.Record.organization_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_id', full_name='public.Record.model_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prediction_id', full_name='public.Record.prediction_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prediction', full_name='public.Record.prediction', index=3,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actual', full_name='public.Record.actual', index=4,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_importances', full_name='public.Record.feature_importances', index=5,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prediction_and_actual', full_name='public.Record.prediction_and_actual', index=6,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=478,
)


_PREPRODUCTIONRECORD_VALIDATIONRECORD = _descriptor.Descriptor(
  name='ValidationRecord',
  full_name='public.PreProductionRecord.ValidationRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='batch_id', full_name='public.PreProductionRecord.ValidationRecord.batch_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='record', full_name='public.PreProductionRecord.ValidationRecord.record', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=650,
  serialized_end=718,
)

_PREPRODUCTIONRECORD_TRAININGRECORD = _descriptor.Descriptor(
  name='TrainingRecord',
  full_name='public.PreProductionRecord.TrainingRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='record', full_name='public.PreProductionRecord.TrainingRecord.record', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=720,
  serialized_end=768,
)

_PREPRODUCTIONRECORD = _descriptor.Descriptor(
  name='PreProductionRecord',
  full_name='public.PreProductionRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='training_record', full_name='public.PreProductionRecord.training_record', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validation_record', full_name='public.PreProductionRecord.validation_record', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_PREPRODUCTIONRECORD_VALIDATIONRECORD, _PREPRODUCTIONRECORD_TRAININGRECORD, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='record_type', full_name='public.PreProductionRecord.record_type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=481,
  serialized_end=783,
)


_SCORECATEGORICAL = _descriptor.Descriptor(
  name='ScoreCategorical',
  full_name='public.ScoreCategorical',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='categorical', full_name='public.ScoreCategorical.categorical', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='score', full_name='public.ScoreCategorical.score', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=785,
  serialized_end=839,
)


_LABEL = _descriptor.Descriptor(
  name='Label',
  full_name='public.Label',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='binary', full_name='public.Label.binary', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='categorical', full_name='public.Label.categorical', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='numeric', full_name='public.Label.numeric', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='score_categorical', full_name='public.Label.score_categorical', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='public.Label.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=842,
  serialized_end=972,
)


_PREDICTION_FEATURESENTRY = _descriptor.Descriptor(
  name='FeaturesEntry',
  full_name='public.Prediction.FeaturesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='public.Prediction.FeaturesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='public.Prediction.FeaturesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1141,
  serialized_end=1203,
)

_PREDICTION = _descriptor.Descriptor(
  name='Prediction',
  full_name='public.Prediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='public.Prediction.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_version', full_name='public.Prediction.model_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label', full_name='public.Prediction.label', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='public.Prediction.features', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_PREDICTION_FEATURESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=975,
  serialized_end=1203,
)


_VALUE = _descriptor.Descriptor(
  name='Value',
  full_name='public.Value',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='string', full_name='public.Value.string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int', full_name='public.Value.int', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double', full_name='public.Value.double', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multi_value', full_name='public.Value.multi_value', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='public.Value.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1205,
  serialized_end=1314,
)


_MULTIVALUE = _descriptor.Descriptor(
  name='MultiValue',
  full_name='public.MultiValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='public.MultiValue.values', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1316,
  serialized_end=1344,
)


_ACTUAL = _descriptor.Descriptor(
  name='Actual',
  full_name='public.Actual',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='public.Actual.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label', full_name='public.Actual.label', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1346,
  serialized_end=1431,
)


_FEATUREIMPORTANCES_FEATUREIMPORTANCESENTRY = _descriptor.Descriptor(
  name='FeatureImportancesEntry',
  full_name='public.FeatureImportances.FeatureImportancesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='public.FeatureImportances.FeatureImportancesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='public.FeatureImportances.FeatureImportancesEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1607,
  serialized_end=1664,
)

_FEATUREIMPORTANCES = _descriptor.Descriptor(
  name='FeatureImportances',
  full_name='public.FeatureImportances',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='public.FeatureImportances.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_version', full_name='public.FeatureImportances.model_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_importances', full_name='public.FeatureImportances.feature_importances', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_FEATUREIMPORTANCES_FEATUREIMPORTANCESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1434,
  serialized_end=1664,
)


_PREDICTIONANDACTUAL = _descriptor.Descriptor(
  name='PredictionAndActual',
  full_name='public.PredictionAndActual',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='prediction', full_name='public.PredictionAndActual.prediction', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actual', full_name='public.PredictionAndActual.actual', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1666,
  serialized_end=1759,
)


_FILEHEADER = _descriptor.Descriptor(
  name='FileHeader',
  full_name='public.FileHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='environment', full_name='public.FileHeader.environment', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FILEHEADER_ENVIRONMENT,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1762,
  serialized_end=1901,
)

_BULKRECORD.fields_by_name['records'].message_type = _RECORD
_RECORD.fields_by_name['prediction'].message_type = _PREDICTION
_RECORD.fields_by_name['actual'].message_type = _ACTUAL
_RECORD.fields_by_name['feature_importances'].message_type = _FEATUREIMPORTANCES
_RECORD.fields_by_name['prediction_and_actual'].message_type = _PREDICTIONANDACTUAL
_PREPRODUCTIONRECORD_VALIDATIONRECORD.fields_by_name['record'].message_type = _RECORD
_PREPRODUCTIONRECORD_VALIDATIONRECORD.containing_type = _PREPRODUCTIONRECORD
_PREPRODUCTIONRECORD_TRAININGRECORD.fields_by_name['record'].message_type = _RECORD
_PREPRODUCTIONRECORD_TRAININGRECORD.containing_type = _PREPRODUCTIONRECORD
_PREPRODUCTIONRECORD.fields_by_name['training_record'].message_type = _PREPRODUCTIONRECORD_TRAININGRECORD
_PREPRODUCTIONRECORD.fields_by_name['validation_record'].message_type = _PREPRODUCTIONRECORD_VALIDATIONRECORD
_PREPRODUCTIONRECORD.oneofs_by_name['record_type'].fields.append(
  _PREPRODUCTIONRECORD.fields_by_name['training_record'])
_PREPRODUCTIONRECORD.fields_by_name['training_record'].containing_oneof = _PREPRODUCTIONRECORD.oneofs_by_name['record_type']
_PREPRODUCTIONRECORD.oneofs_by_name['record_type'].fields.append(
  _PREPRODUCTIONRECORD.fields_by_name['validation_record'])
_PREPRODUCTIONRECORD.fields_by_name['validation_record'].containing_oneof = _PREPRODUCTIONRECORD.oneofs_by_name['record_type']
_LABEL.fields_by_name['score_categorical'].message_type = _SCORECATEGORICAL
_LABEL.oneofs_by_name['data'].fields.append(
  _LABEL.fields_by_name['binary'])
_LABEL.fields_by_name['binary'].containing_oneof = _LABEL.oneofs_by_name['data']
_LABEL.oneofs_by_name['data'].fields.append(
  _LABEL.fields_by_name['categorical'])
_LABEL.fields_by_name['categorical'].containing_oneof = _LABEL.oneofs_by_name['data']
_LABEL.oneofs_by_name['data'].fields.append(
  _LABEL.fields_by_name['numeric'])
_LABEL.fields_by_name['numeric'].containing_oneof = _LABEL.oneofs_by_name['data']
_LABEL.oneofs_by_name['data'].fields.append(
  _LABEL.fields_by_name['score_categorical'])
_LABEL.fields_by_name['score_categorical'].containing_oneof = _LABEL.oneofs_by_name['data']
_PREDICTION_FEATURESENTRY.fields_by_name['value'].message_type = _VALUE
_PREDICTION_FEATURESENTRY.containing_type = _PREDICTION
_PREDICTION.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_PREDICTION.fields_by_name['label'].message_type = _LABEL
_PREDICTION.fields_by_name['features'].message_type = _PREDICTION_FEATURESENTRY
_VALUE.fields_by_name['multi_value'].message_type = _MULTIVALUE
_VALUE.oneofs_by_name['data'].fields.append(
  _VALUE.fields_by_name['string'])
_VALUE.fields_by_name['string'].containing_oneof = _VALUE.oneofs_by_name['data']
_VALUE.oneofs_by_name['data'].fields.append(
  _VALUE.fields_by_name['int'])
_VALUE.fields_by_name['int'].containing_oneof = _VALUE.oneofs_by_name['data']
_VALUE.oneofs_by_name['data'].fields.append(
  _VALUE.fields_by_name['double'])
_VALUE.fields_by_name['double'].containing_oneof = _VALUE.oneofs_by_name['data']
_VALUE.oneofs_by_name['data'].fields.append(
  _VALUE.fields_by_name['multi_value'])
_VALUE.fields_by_name['multi_value'].containing_oneof = _VALUE.oneofs_by_name['data']
_ACTUAL.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ACTUAL.fields_by_name['label'].message_type = _LABEL
_FEATUREIMPORTANCES_FEATUREIMPORTANCESENTRY.containing_type = _FEATUREIMPORTANCES
_FEATUREIMPORTANCES.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FEATUREIMPORTANCES.fields_by_name['feature_importances'].message_type = _FEATUREIMPORTANCES_FEATUREIMPORTANCESENTRY
_PREDICTIONANDACTUAL.fields_by_name['prediction'].message_type = _PREDICTION
_PREDICTIONANDACTUAL.fields_by_name['actual'].message_type = _ACTUAL
_FILEHEADER.fields_by_name['environment'].enum_type = _FILEHEADER_ENVIRONMENT
_FILEHEADER_ENVIRONMENT.containing_type = _FILEHEADER
DESCRIPTOR.message_types_by_name['BulkRecord'] = _BULKRECORD
DESCRIPTOR.message_types_by_name['Record'] = _RECORD
DESCRIPTOR.message_types_by_name['PreProductionRecord'] = _PREPRODUCTIONRECORD
DESCRIPTOR.message_types_by_name['ScoreCategorical'] = _SCORECATEGORICAL
DESCRIPTOR.message_types_by_name['Label'] = _LABEL
DESCRIPTOR.message_types_by_name['Prediction'] = _PREDICTION
DESCRIPTOR.message_types_by_name['Value'] = _VALUE
DESCRIPTOR.message_types_by_name['MultiValue'] = _MULTIVALUE
DESCRIPTOR.message_types_by_name['Actual'] = _ACTUAL
DESCRIPTOR.message_types_by_name['FeatureImportances'] = _FEATUREIMPORTANCES
DESCRIPTOR.message_types_by_name['PredictionAndActual'] = _PREDICTIONANDACTUAL
DESCRIPTOR.message_types_by_name['FileHeader'] = _FILEHEADER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BulkRecord = _reflection.GeneratedProtocolMessageType('BulkRecord', (_message.Message,), {
  'DESCRIPTOR' : _BULKRECORD,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.BulkRecord)
  })
_sym_db.RegisterMessage(BulkRecord)

Record = _reflection.GeneratedProtocolMessageType('Record', (_message.Message,), {
  'DESCRIPTOR' : _RECORD,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.Record)
  })
_sym_db.RegisterMessage(Record)

PreProductionRecord = _reflection.GeneratedProtocolMessageType('PreProductionRecord', (_message.Message,), {

  'ValidationRecord' : _reflection.GeneratedProtocolMessageType('ValidationRecord', (_message.Message,), {
    'DESCRIPTOR' : _PREPRODUCTIONRECORD_VALIDATIONRECORD,
    '__module__' : 'public_pb2'
    # @@protoc_insertion_point(class_scope:public.PreProductionRecord.ValidationRecord)
    })
  ,

  'TrainingRecord' : _reflection.GeneratedProtocolMessageType('TrainingRecord', (_message.Message,), {
    'DESCRIPTOR' : _PREPRODUCTIONRECORD_TRAININGRECORD,
    '__module__' : 'public_pb2'
    # @@protoc_insertion_point(class_scope:public.PreProductionRecord.TrainingRecord)
    })
  ,
  'DESCRIPTOR' : _PREPRODUCTIONRECORD,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.PreProductionRecord)
  })
_sym_db.RegisterMessage(PreProductionRecord)
_sym_db.RegisterMessage(PreProductionRecord.ValidationRecord)
_sym_db.RegisterMessage(PreProductionRecord.TrainingRecord)

ScoreCategorical = _reflection.GeneratedProtocolMessageType('ScoreCategorical', (_message.Message,), {
  'DESCRIPTOR' : _SCORECATEGORICAL,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.ScoreCategorical)
  })
_sym_db.RegisterMessage(ScoreCategorical)

Label = _reflection.GeneratedProtocolMessageType('Label', (_message.Message,), {
  'DESCRIPTOR' : _LABEL,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.Label)
  })
_sym_db.RegisterMessage(Label)

Prediction = _reflection.GeneratedProtocolMessageType('Prediction', (_message.Message,), {

  'FeaturesEntry' : _reflection.GeneratedProtocolMessageType('FeaturesEntry', (_message.Message,), {
    'DESCRIPTOR' : _PREDICTION_FEATURESENTRY,
    '__module__' : 'public_pb2'
    # @@protoc_insertion_point(class_scope:public.Prediction.FeaturesEntry)
    })
  ,
  'DESCRIPTOR' : _PREDICTION,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.Prediction)
  })
_sym_db.RegisterMessage(Prediction)
_sym_db.RegisterMessage(Prediction.FeaturesEntry)

Value = _reflection.GeneratedProtocolMessageType('Value', (_message.Message,), {
  'DESCRIPTOR' : _VALUE,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.Value)
  })
_sym_db.RegisterMessage(Value)

MultiValue = _reflection.GeneratedProtocolMessageType('MultiValue', (_message.Message,), {
  'DESCRIPTOR' : _MULTIVALUE,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.MultiValue)
  })
_sym_db.RegisterMessage(MultiValue)

Actual = _reflection.GeneratedProtocolMessageType('Actual', (_message.Message,), {
  'DESCRIPTOR' : _ACTUAL,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.Actual)
  })
_sym_db.RegisterMessage(Actual)

FeatureImportances = _reflection.GeneratedProtocolMessageType('FeatureImportances', (_message.Message,), {

  'FeatureImportancesEntry' : _reflection.GeneratedProtocolMessageType('FeatureImportancesEntry', (_message.Message,), {
    'DESCRIPTOR' : _FEATUREIMPORTANCES_FEATUREIMPORTANCESENTRY,
    '__module__' : 'public_pb2'
    # @@protoc_insertion_point(class_scope:public.FeatureImportances.FeatureImportancesEntry)
    })
  ,
  'DESCRIPTOR' : _FEATUREIMPORTANCES,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.FeatureImportances)
  })
_sym_db.RegisterMessage(FeatureImportances)
_sym_db.RegisterMessage(FeatureImportances.FeatureImportancesEntry)

PredictionAndActual = _reflection.GeneratedProtocolMessageType('PredictionAndActual', (_message.Message,), {
  'DESCRIPTOR' : _PREDICTIONANDACTUAL,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.PredictionAndActual)
  })
_sym_db.RegisterMessage(PredictionAndActual)

FileHeader = _reflection.GeneratedProtocolMessageType('FileHeader', (_message.Message,), {
  'DESCRIPTOR' : _FILEHEADER,
  '__module__' : 'public_pb2'
  # @@protoc_insertion_point(class_scope:public.FileHeader)
  })
_sym_db.RegisterMessage(FileHeader)


DESCRIPTOR._options = None
_PREDICTION_FEATURESENTRY._options = None
_FEATUREIMPORTANCES_FEATUREIMPORTANCESENTRY._options = None
# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod

import pandas as pd
from typing import Optional, Union, Dict, List

from arize import public_pb2 as public__pb2
from arize.utils.types import ModelTypes
from arize.utils.utils import (
    validate_prediction_timestamps,
    bundle_records,
    convert_element,
    get_value_object,
    get_timestamp,
    infer_model_type,
    get_bulk_records,
)

# TODO this class hierarchy will go away


class BaseRecord(ABC):
    def __init__(
        self,
        organization_key: str,
        model_id: str,
        model_type: Optional[ModelTypes] = None,
    ):
        self.organization_key = organization_key
        self.model_id = model_id
        self.model_type = model_type

    @abstractmethod
    def validate_inputs(self):
        pass

    @abstractmethod
    def build_proto(self):
        pass

    def _base_validation(self):
        if not isinstance(self.organization_key, str):
            raise TypeError(
                f"organization_key {self.organization_key} is type {type(self.organization_key)}, but must be a str"
            )
        if not isinstance(self.model_id, str):
            raise TypeError(
                f"model_id {self.model_id} is type {type(self.model_id)}, but must be a str"
            )

    def _label_validation(self, label):
        if self.model_type == ModelTypes.BINARY:
            if not (isinstance(label, bool) or label == 0 or label == 1):
                raise TypeError(
                    f"label {label} has type {type(label)}, but must be one a bool, 0 or 1 for ModelTypes.BINARY"
                )
        elif self.model_type == ModelTypes.NUMERIC:
            if not isinstance(label, (float, int)):
                raise TypeError(
                    f"label {label} has type {type(label)}, but must be either float or int for ModelTypes.NUMERIC"
                )
        elif self.model_type == ModelTypes.CATEGORICAL:
            if not isinstance(label, str):
                raise TypeError(
                    f"label {label} has type {type(label)}, but must be str for ModelTypes.CATEGORICAL"
                )
        elif self.model_type == ModelTypes.SCORE_CATEGORICAL:
            if not isinstance(label, str):
                raise TypeError(
                    f"label {label} has type {type(label)}, but must be str for ModelTypes.SCORE_CATEGORICAL"
                )

    def _get_label(
        self, name: str, value, score: Optional[float] = None
    ) -> public__pb2.Label:
        if isinstance(value, public__pb2.Label):
            return value
        val = convert_element(value)
        if self.model_type == ModelTypes.SCORE_CATEGORICAL:
            return public__pb2.Label(
                score_categorical=public__pb2.ScoreCategorical(
                    categorical=val, score=convert_element(score)
                )
            )
        elif self.model_type == ModelTypes.BINARY:
            return public__pb2.Label(binary=val)
        elif self.model_type == ModelTypes.NUMERIC:
            return public__pb2.Label(numeric=val)
        elif self.model_type == ModelTypes.CATEGORICAL:
            return public__pb2.Label(categorical=val)
        raise TypeError(
            f"{name}_label = {value} of type {type(value)}. Must be one of str, bool, float, or int"
        )


class PreProductionRecords(BaseRecord, ABC):
    def __init__(
        self,
        organization_key: str,
        model_id: str,
        model_version: str,
        prediction_labels: Union[pd.DataFrame, pd.Series],
        actual_labels: Union[pd.DataFrame, pd.Series],
        features: Optional[pd.DataFrame] = None,
        model_type: Optional[ModelTypes] = None,
        prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
        prediction_ids: Optional[Union[pd.DataFrame, pd.Series]] = None,
    ):
        if model_type is None:
            if prediction_scores is None:
                model_type = (
                    infer_model_type(prediction_labels[0])
                    if model_type is None
                    else model_type
                )
            else:
                model_type = ModelTypes.SCORE_CATEGORICAL
        super().__init__(
            organization_key=organization_key, model_id=model_id, model_type=model_type
        )
        self.model_version = model_version
        self.features = features
        self.prediction_labels = prediction_labels
        self.prediction_scores = prediction_scores
        self.actual_labels = actual_labels
        self.prediction_ids = prediction_ids

    def _validate_preprod_inputs(self):
        self._base_validation()
        if not isinstance(self.model_version, str):
            raise TypeError(
                f"model_version {self.model_version} is type {type(self.model_version)}, but must be a str"
            )
        if not isinstance(self.prediction_labels, (pd.DataFrame, pd.Series)):
            raise TypeError(
                f"prediction_labels is type {type(self.prediction_labels)}, but expects one of: pd.DataFrame, pd.Series"
            )
        if self.prediction_labels.shape[0] != self.actual_labels.shape[0]:
            raise ValueError(
                f"prediction_labels contains {self.prediction_labels.shape[0]} elements, but must have the same as "
                f"actual_labels: {self.actual_labels.shape[0]}. "
            )
        if self.model_type == ModelTypes.SCORE_CATEGORICAL:
            if not isinstance(self.prediction_scores, (pd.DataFrame, pd.Series)):
                raise TypeError(
                    f"prediction_scores is type {type(self.prediction_scores)}, but expects one of: pd.DataFrame, pd.Series"
                )
            if self.prediction_scores.shape[0] != self.prediction_labels.shape[0]:
                raise ValueError(
                    f"prediction_scores contains {self.prediction_scores.shape[0]} elements, but must have the same as "
                    f"prediction_labels: {self.prediction_scores.shape[0]}."
                )

        if self.prediction_ids is not None:
            if not isinstance(self.prediction_labels, (pd.DataFrame, pd.Series)):
                raise TypeError(
                    f"prediction_ids is type {type(self.prediction_ids)}, but expects one of: pd.DataFrame, pd.Series"
                )
            if self.prediction_labels.shape[0] != self.prediction_ids.shape[0]:
                raise ValueError(
                    f"prediction_ids contains {self.prediction_ids.shape[0]} elements, but must have the same as "
                    f"prediction_labels: {self.prediction_labels.shape[0]}. "
                )

        if self.features is None:
            return
        if not isinstance(self.features, pd.DataFrame):
            raise TypeError(
                f"features is type {type(self.features)}, but expect type pd.DataFrame."
            )
        if self.features.shape[0] != self.prediction_labels.shape[0]:
            raise ValueError(
                f"features has {self.features.shape[0]} sets of features, but must match size of prediction_labels: "
                f"{self.prediction_labels.shape[0]}. "
            )
        for name in self.features.columns:
            if not isinstance(name, str):
                raise TypeError(
                    f"features.column {name} is type {type(name)}, but expect str"
                )

    def _normalize_inputs(self):
        """Converts inputs from DataFrames, Series, lists to numpy arrays or lists for consistent iterations
        downstream."""
        if isinstance(self.prediction_labels, (pd.DataFrame, pd.Series)):
            self.prediction_labels = self.prediction_labels.to_numpy()
        if isinstance(self.prediction_ids, (pd.DataFrame, pd.Series)):
            self.prediction_ids = self.prediction_ids.to_numpy()
        if isinstance(self.actual_labels, (pd.DataFrame, pd.Series)):
            self.actual_labels = self.actual_labels.to_numpy()
        if isinstance(self.prediction_scores, (pd.DataFrame, pd.Series)):
            self.prediction_scores = self.prediction_scores.to_numpy()
        if isinstance(self.features, pd.DataFrame):
            self.feature_names = self.features.columns
            self.features = self.features.to_numpy()


class TrainingRecords(PreProductionRecords):
    def __init__(
        self,
        organization_key: str,
        model_id: str,
        model_version: str,
        prediction_labels: Union[pd.DataFrame, pd.Series],
        actual_labels: Union[pd.DataFrame, pd.Series],
        features: Optional[pd.DataFrame] = None,
        model_type: Optional[ModelTypes] = None,
        prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
    ):
        super().__init__(
            organization_key=organization_key,
            model_id=model_id,
            model_type=model_type,
            model_version=model_version,
            features=features,
            prediction_labels=prediction_labels,
            prediction_scores=prediction_scores,
            actual_labels=actual_labels,
        )

    def validate_inputs(self):
        self._validate_preprod_inputs()

    def build_proto(self):
        records = []
        self._normalize_inputs()
        for row, v in enumerate(self.prediction_labels):
            a = public__pb2.Actual(
                label=self._get_label(value=self.actual_labels[row], name="actual")
            )
            score = (
                None
                if self.model_type != ModelTypes.SCORE_CATEGORICAL
                else self.prediction_scores[row]
            )
            p = public__pb2.Prediction(
                label=self._get_label(value=v, name="prediction", score=score),
                model_version=self.model_version,
            )

            if self.features is not None:
                converted_feats = {}
                for column, name in enumerate(self.feature_names):
                    val = get_value_object(value=self.features[row][column], name=name)
                    if val is not None:
                        converted_feats[name] = val
                feats = public__pb2.Prediction(features=converted_feats)
                p.MergeFrom(feats)

            panda = public__pb2.PredictionAndActual(
                prediction=p,
                actual=a,
            )
            r = public__pb2.Record(
                organization_key=self.organization_key,
                model_id=self.model_id,
                prediction_and_actual=panda,
            )

            t = public__pb2.PreProductionRecord(
                training_record=public__pb2.PreProductionRecord.TrainingRecord(record=r)
            )
            records.append(t)
        return bundle_records(records)


class ValidationRecords(PreProductionRecords):
    def __init__(
        self,
        organization_key: str,
        model_id: str,
        model_version: str,
        batch_id: str,
        prediction_labels: Union[pd.DataFrame, pd.Series],
        actual_labels: Union[pd.DataFrame, pd.Series],
        features: Optional[pd.DataFrame] = None,
        model_type: Optional[ModelTypes] = None,
        prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
        prediction_ids: Optional[Union[pd.DataFrame, pd.Series]] = None,
        prediction_timestamps: Optional[Union[List[int], pd.Series]] = None,
    ):
        super().__init__(
            organization_key=organization_key,
            model_id=model_id,
            model_type=model_type,
            model_version=model_version,
            features=features,
            prediction_labels=prediction_labels,
            prediction_scores=prediction_scores,
            actual_labels=actual_labels,
            prediction_ids=prediction_ids,
        )
        self.batch_id = batch_id
        self.prediction_timestamps = prediction_timestamps

    def validate_inputs(self):
        self._validate_preprod_inputs()
        if not isinstance(self.batch_id, str):
            raise TypeError(
                f"batch_id {self.batch_id} is type {type(self.batch_id)}, but must be a str"
            )
        validate_prediction_timestamps(self.prediction_ids, self.prediction_timestamps)

    def build_proto(self):
        records = []
        self._normalize_inputs()
        prediction_timestamps = (
            self.prediction_timestamps.tolist()
            if isinstance(self.prediction_timestamps, pd.Series)
            else self.prediction_timestamps
        )
        for row, v in enumerate(self.prediction_labels):
            a = public__pb2.Actual(
                label=self._get_label(value=self.actual_labels[row], name="actual")
            )
            score = (
                None
                if self.model_type != ModelTypes.SCORE_CATEGORICAL
                else self.prediction_scores[row]
            )
            p = public__pb2.Prediction(
                label=self._get_label(value=v, name="prediction", score=score),
                model_version=self.model_version,
            )

            if prediction_timestamps is not None:
                p.timestamp.MergeFrom(get_timestamp(prediction_timestamps[row]))

            prediction_id = None
            if self.prediction_ids is not None:
                v = self.prediction_ids[row]
                prediction_id = v if isinstance(v, str) else v[0]
                if not isinstance(prediction_id, str):
                    raise TypeError(
                        f"prediction_id={prediction_id} of type {type(prediction_id)}. Must be str type."
                    )

            if self.features is not None:
                converted_feats = {}
                for column, name in enumerate(self.feature_names):
                    val = get_value_object(value=self.features[row][column], name=name)
                    if val is not None:
                        converted_feats[name] = val
                feats = public__pb2.Prediction(features=converted_feats)
                p.MergeFrom(feats)

            panda = public__pb2.PredictionAndActual(
                prediction=p,
                actual=a,
            )
            r = public__pb2.Record(
                prediction_id=prediction_id,
                organization_key=self.organization_key,
                model_id=self.model_id,
                prediction_and_actual=panda,
            )
            v = public__pb2.PreProductionRecord(
                validation_record=public__pb2.PreProductionRecord.ValidationRecord(
                    batch_id=self.batch_id, record=r
                ),
            )
            records.append(v)
        return bundle_records(records)



import os
import time
import uuid

import pandas as pd
import numpy as np
import concurrent.futures as cf

from arize.api import Client
from arize.types import ModelTypes

MODELS = 1
VERSIONS = 1
BATCHES = 1
NUM_RECORDS = 1

arize = Client(
    organization_key=os.environ.get("ARIZE_ORG_KEY"),
    api_key=os.environ.get("ARIZE_API_KEY"),
)

features = pd.DataFrame(
    np.random.randint(0, 100000000, size=(NUM_RECORDS, 12)),
    columns=list("ABCDEFGHIJKL"),
)
pred_labels = pd.DataFrame(np.random.randint(0, 100000000, size=(NUM_RECORDS, 1)))
actual_labels = pd.DataFrame(np.random.randint(0, 100000000, size=(NUM_RECORDS, 1)))

ids = pd.DataFrame([str(uuid.uuid4()) for _ in range(NUM_RECORDS)])
start = time.time_ns()

futures = []
for m in range(MODELS):
    model_id = f"multi_env_model_{m}"
    for v in range(VERSIONS):
        model_version = f"v0.{v}"
        preds = arize.log_bulk_predictions(
            model_id=model_id,
            model_type=ModelTypes.NUMERIC,
            model_version=model_version,
            prediction_ids=ids,
            prediction_labels=pred_labels,
            features=features,
        )
        futures.extend(preds)
        actuals = arize.log_bulk_actuals(
            model_id=model_id,
            prediction_ids=ids,
            actual_labels=actual_labels,
            model_type=ModelTypes.NUMERIC,
        )
        futures.extend(actuals)
        tFuture = arize.log_training_records(
            model_id=model_id,
            model_type=ModelTypes.NUMERIC,
            model_version=model_version,
            prediction_labels=pred_labels,
            actual_labels=actual_labels,
            features=features,
        )
        futures.extend(tFuture)
        for b in range(BATCHES):
            batch = f"batch-{b}"
            vFuture = arize.log_validation_records(
                model_id=model_id,
                model_type=ModelTypes.NUMERIC,
                model_version=model_version,
                batch_id=batch,
                prediction_labels=pred_labels,
                actual_labels=actual_labels,
                features=features,
            )
            futures.extend(vFuture)


end_enqueue = time.time_ns()
print(
    f"request took a total of {int(end_enqueue - start)/1000000}ms to queue. Waiting for response."
)

for future in cf.as_completed(futures):
    res = future.result()
    print(f"future completed with response code {res.status_code}")
    if res.status_code != 200:
        print(f"future failed with response code {res.status_code}, {res.text}")

end_resp = time.time_ns()
print(f"Total time: {int(end_resp - start)/1000000}ms")



from setuptools import setup, find_packages
from codecs import open

import sys, os


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), encoding='utf-8') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


if sys.version_info < (3, 5, 3):
    sys.exit('Sorry, Python < 3.5.3 is not supported')

long_description = read('README.md')

# get the dependencies and installs
all_reqs = read('requirements.txt').splitlines()

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
dependency_links = [
    x.strip().replace('git+', '') for x in all_reqs if x.startswith('git+')
]

test_reqs = read('requirements-dev.txt').splitlines()
test_requirements = [x.strip() for x in test_reqs if 'git+' not in x]

__version__ = get_version("arize/__init__.py")

setup(
    name='arize',
    version=__version__,
    description='A helper library to interact with Arize AI APIs',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Arize-ai/client_python',
    project_urls={
        'Arize AI': 'https://www.arize.com',
    },
    download_url='https://github.com/Arize-ai/client_python/tarball/' +
    __version__,
    author='Arize AI',
    author_email='support@arize.com',
    license='BSD',
    python_requires='>=3.5.3',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='arize',
    packages=find_packages(exclude=['docs', 'tests*']),
    include_package_data=True,
    install_requires=install_requires,
    tests_require=test_requirements,
    dependency_links=dependency_links,
)
